@@ -753,8 +753,8 @@ typedef struct
 	unsigned char	led_ctrl;
 	unsigned int	tdm_intr_status;
 	void 		*bar_virt;
-	unsigned short	tdm_rx_dma_toggle;
-	unsigned short	tdm_tx_dma_toggle;
+	unsigned char	tdm_rx_dma_toggle[32];
+	unsigned char	tdm_tx_dma_toggle[32];
 	unsigned int	tdm_logic_ch_map;
 
 	wan_ticks_t	sec_chk_cnt;
@@ -771,7 +771,8 @@ enum {
 	AFT_TDM_GLOBAL_ISR,
 	AFT_TDM_RING_BUF,
 	AFT_TDM_FAST_ISR,
-	AFT_TDM_SW_RING_BUF
+	AFT_TDM_SW_RING_BUF,
+	AFT_TDM_FREE_RUN_ISR
 };
 
 typedef struct 
@@ -980,11 +981,16 @@ typedef struct sdla
 
 	/* SDLA TDMV Dummy interface */
 #if defined(CONFIG_PRODUCT_WANPIPE_TDM_VOICE)
-        void* sdla_tdmv_dummy;
+	void* sdla_tdmv_dummy;
 #endif
 
 	unsigned char wp_debug_chan_seq;
 
+#if defined(WANPIPE_PERFORMANCE_DEBUG)
+ 	wan_ticks_t				debug_timeout;
+	struct timeval			timing_tv;
+#endif
+
 } sdla_t;
 
 /****** Public Functions ****************************************************/
