@@ -244,7 +244,7 @@ static int wp_tdmv_remora_rx_chan_sync_test(sdla_t *card, wp_tdmv_remora_t *wr,
 
 #define wp_fax_tone_timeout_set(wr,chan) do { DEBUG_TEST("%s:%d: s%dc%d fax timeout set\n", \
 											__FUNCTION__,__LINE__, \
-											wr->spanno+1,chan); \
+											wr->spanno+1,chan+1); \
 											wr->ec_fax_detect_timeout[chan]=SYSTEM_TICKS; } while(0);
 
 static int
@@ -413,7 +413,7 @@ static int wp_remora_zap_hooksig(struct zt_chan *chan, zt_txsig_t txsig)
 	wp_tdmv_remora_t	*wr = chan->pvt;
 	sdla_t			*card = NULL;
 	sdla_fe_t		*fe = NULL;
-	int			fe_chan = chan->chanpos;
+	int			fe_chan = chan->chanpos-1;
 
 	WAN_ASSERT(wr->card == NULL);
 	card	= wr->card;
@@ -510,12 +510,16 @@ static int wp_remora_zap_hooksig(struct zt_chan *chan, zt_txsig_t txsig)
 static int wp_remora_zap_open(struct zt_chan *chan)
 {
 	wp_tdmv_remora_t	*wr = NULL;
+	sdla_t		*card = NULL; 
 
 	WAN_ASSERT2(chan == NULL, -ENODEV);
 	WAN_ASSERT2(chan->pvt == NULL, -ENODEV);
 	wr = chan->pvt;
+	WAN_ASSERT2(wr->card == NULL, -ENODEV);
+    card = wr->card; 
 	wr->usecount++;
 	wan_set_bit(WP_TDMV_RUNNING, &wr->flags);
+	wanpipe_open(card);
 	DEBUG_EVENT("%s: Open (usecount=%d, channo=%d, chanpos=%d)...\n", 
 				wr->devname,
 				wr->usecount,
@@ -536,6 +540,7 @@ static int wp_remora_zap_close(struct zt_chan *chan)
 	card	= wr->card;
 	fe	= &card->fe;
 	wr->usecount--;
+	wanpipe_close(card);   
 	wan_clear_bit(WP_TDMV_RUNNING, &wr->flags);
 
 #if 1
@@ -547,7 +552,8 @@ static int wp_remora_zap_close(struct zt_chan *chan)
 	}
 #endif
 	return 0;
-}
+}           
+
 
 static int wp_remora_zap_watchdog(struct zt_span *span, int event)
 {
@@ -570,7 +576,6 @@ static int wp_tdmv_remora_hwec_create(struct dahdi_chan *chan, struct dahdi_echo
 {
 	wp_tdmv_remora_t *wr = NULL;
 	sdla_t *card = NULL;
-	int fe_chan = chan->chanpos;
 	int err = -ENODEV;
 
 	WAN_ASSERT2(chan == NULL, -ENODEV);
@@ -584,11 +589,11 @@ static int wp_tdmv_remora_hwec_create(struct dahdi_chan *chan, struct dahdi_echo
 		return -EINVAL;
 	}
 
-	*ec = &wr->ec[fe_chan];
+	*ec = &wr->ec[chan->chanpos-1];
 	(*ec)->ops = &wp_tdmv_remora_ec_ops;
 	(*ec)->features = wp_tdmv_remora_ec_features;
 
-	wan_set_bit(fe_chan, &card->wandev.rtp_tap_call_map);
+	wan_set_bit(chan->chanpos-1, &card->wandev.rtp_tap_call_map);
 
 	if (card->wandev.ec_enable) {
 		/* The ec persist flag enables and disables
@@ -597,12 +602,12 @@ static int wp_tdmv_remora_hwec_create(struct dahdi_chan *chan, struct dahdi_echo
                  * asterisk.  In persist mode off asterisk 
                  * controls hardware echo cancellation */		 
 		if (card->hwec_conf.persist_disable) {
-			err = card->wandev.ec_enable(card, 1, fe_chan);
+			err = card->wandev.ec_enable(card, 1, chan->chanpos);
 		} else {
 			err = 0;			
 		}           
 		DEBUG_TDMV("[TDMV_RM]: %s: Enable HW echo canceller on channel %d\n",        
-				wr->devname, fe_chan);
+				wr->devname, chan->chanpos);
 	}
 	return err;
 }
@@ -617,7 +622,6 @@ static void wp_tdmv_remora_hwec_free(struct dahdi_chan *chan, struct dahdi_echoc
 {
 	wp_tdmv_remora_t *wr = NULL;
 	sdla_t *card = NULL;
-	int	fe_chan = chan->chanpos;
 
 	memset(ec, 0, sizeof(*ec));
 
@@ -627,7 +631,7 @@ static void wp_tdmv_remora_hwec_free(struct dahdi_chan *chan, struct dahdi_echoc
 	if(wr->card == NULL) return;
 	card = wr->card;
 
-	wan_clear_bit(fe_chan, &card->wandev.rtp_tap_call_map);
+	wan_clear_bit(chan->chanpos-1, &card->wandev.rtp_tap_call_map);
 
 	if (card->wandev.ec_enable) {
 		/* The ec persist flag enables and disables
@@ -636,10 +640,10 @@ static void wp_tdmv_remora_hwec_free(struct dahdi_chan *chan, struct dahdi_echoc
                  * asterisk.  In persist mode off asterisk 
                  * controls hardware echo cancellation */
 		if (card->hwec_conf.persist_disable) {
-			card->wandev.ec_enable(card, 0, fe_chan);
+			card->wandev.ec_enable(card, 0, chan->chanpos);
 		}
 		DEBUG_TDMV("[TDMV] %s: Disable HW echo canceller on channel %d\n",
-				wr->devname, fe_chan);
+				wr->devname, chan->chanpos);
 	}
 }
 
@@ -654,7 +658,6 @@ static int wp_remora_zap_hwec(struct zt_chan *chan, int enable)
 {
 	wp_tdmv_remora_t    *wr = NULL;
 	sdla_t          *card = NULL;
-	int         fe_chan = chan->chanpos;
 	int         err = -ENODEV;
 
 	WAN_ASSERT2(chan == NULL, -ENODEV);
@@ -664,9 +667,9 @@ static int wp_remora_zap_hwec(struct zt_chan *chan, int enable)
 	card = wr->card;
 
 	if (enable) {
-		wan_set_bit(fe_chan,&card->wandev.rtp_tap_call_map);
+		wan_set_bit(chan->chanpos-1,&card->wandev.rtp_tap_call_map);
 	} else {
-		wan_clear_bit(fe_chan,&card->wandev.rtp_tap_call_map);
+		wan_clear_bit(chan->chanpos-1,&card->wandev.rtp_tap_call_map);
 	}
 
 	if (card->wandev.ec_enable){
@@ -676,14 +679,14 @@ static int wp_remora_zap_hwec(struct zt_chan *chan, int enable)
 		* asterisk.  In persist mode off asterisk
 		* controls hardware echo cancellation */
 		if (card->hwec_conf.persist_disable) {
-			err = card->wandev.ec_enable(card, enable, fe_chan);
+			err = card->wandev.ec_enable(card, enable, chan->chanpos);
 		} else {
 			err = 0;
 		}
 		DEBUG_TDMV("[TDMV_RM]: %s: %s HW echo canceller on channel %d\n",
 				   wr->devname,
 				   (enable) ? "Enable" : "Disable",
-				   fe_chan);
+				   chan->chanpos);
 	}
 	return err;
 }
@@ -1984,12 +1987,12 @@ static int wp_tdmv_remora_ec_span(void *pcard)
 static void wp_tdmv_remora_dtmf (void* card_id, wan_event_t *event)
 {
 	sdla_t	*card = (sdla_t*)card_id;
-        wan_tdmv_t      *wan_tdmv = &card->wan_tdmv;
-        wp_tdmv_remora_t	*wr = NULL;
-	int fechan = event->channel;
+	wan_tdmv_t      *wan_tdmv = &card->wan_tdmv;
+	wp_tdmv_remora_t	*wr = NULL;
+	int fechan = event->channel-1;
 
-        WAN_ASSERT1(wan_tdmv->sc == NULL);
-        wr = wan_tdmv->sc;
+	WAN_ASSERT1(wan_tdmv->sc == NULL);
+	wr = wan_tdmv->sc;
 	
 	if (event->type == WAN_EVENT_EC_DTMF){
 		DEBUG_TDMV(
