@@ -277,7 +277,7 @@ static void wp_tdmv_report_alarms(void* pcard, unsigned long te_alarm);
 /* Rx/Tx functions */
 static int wp_tdmv_rx_tx(void* pcard, netskb_t* skb);
 static int wp_tdmv_rx_tx_span(void *pcard);
-static int wp_tdmv_span_buf_rotate(void *pcard, u32, unsigned long);
+static int wp_tdmv_span_buf_rotate(void *pcard, u32, unsigned long, int);
 static int wp_tdmv_ec_span(void *pcard);
 static int wp_tdmv_rx_chan(wan_tdmv_t*, int, unsigned char*, unsigned char*); 
 #if defined(CONFIG_PRODUCT_WANPIPE_TDM_VOICE_DCHAN) && defined(ZT_DCHAN_TX)
@@ -1472,17 +1472,21 @@ static char *wp_tdmv_sigstr(int sig)
 }
 #endif
 
+
 static int wp_tdmv_chanconfig(struct zt_chan *chan, int sigtype)
 {
 	sdla_t		*card;
 	wp_tdmv_softc_t	*wp = NULL;
+	int hdlcsig = sigtype & ~(ZT_SIG_CLEAR);
 
 	WAN_ASSERT2(chan == NULL, -ENODEV);
 	WAN_ASSERT2(chan->pvt == NULL, -ENODEV);
 	wp	= chan->pvt;
 	card	= (sdla_t*)wp->card;
 
-	DEBUG_TDMV("%s: Configuring chan %d..\n", wp->devname, chan->chanpos);
+	DEBUG_TEST("%s: Configuring chan %d (hdlc=%i) flags=0x%02X sigtype=0x%03X..\n",
+			wp->devname, chan->chanpos, IS_CHAN_HARDHDLC(chan), chan->flags, sigtype);
+
 	if (chan->span->flags & ZT_FLAG_RUNNING){
 		wp_tdmv_set_clear(wp);
 	}
@@ -1498,6 +1502,16 @@ static int wp_tdmv_chanconfig(struct zt_chan *chan, int sigtype)
 		}
 		wp_tdmv_sigctrl(card, wp, chan->chanpos, WP_TDMV_ENABLE);
 	}
+
+	/* Disable hwec on signalling channel */
+	if (wp->hwec == WANOPT_YES && card->wandev.ec_dev && card->wandev.ec_enable) {
+		if (IS_CHAN_HARDHDLC(chan) || hdlcsig & ZT_SIG_HDLCRAW || hdlcsig & ZT_SIG_HDLCFCS) {
+			DEBUG_TEST("%s: Disabling EC Chan=%i, sigtype 0x%03X, HDLCRAW=0x%X  FCS=0x%X\n",
+					card->devname, chan->chanpos, sigtype, ZT_SIG_HDLCRAW, ZT_SIG_HDLCFCS);
+			card->wandev.ec_enable(card, 0, chan->chanpos);
+		}
+	}
+
 	return 0;
 }
 
@@ -2001,7 +2015,6 @@ static int wp_tdmv_hwec_create(struct dahdi_chan *chan, struct dahdi_echocanpara
 {
 	wp_tdmv_softc_t	*wp = NULL;
 	sdla_t *card = NULL;
-	int	channel=0;
 	int	err = -EINVAL;
 
 	if (ecp->param_count > 0) {
@@ -2014,19 +2027,13 @@ static int wp_tdmv_hwec_create(struct dahdi_chan *chan, struct dahdi_echocanpara
 	wp = chan->pvt;
 	WAN_ASSERT2(wp->card == NULL, -ENODEV);
 	card = wp->card;
-	
-	channel = chan->chanpos;
 
-	if (!wp->ise1) {
-		channel--;
-	}
-
-	*ec = &wp->ec[channel];
+	*ec = &wp->ec[chan->chanpos-1];
 	(*ec)->ops = &wp_tdmv_ec_ops;
 	(*ec)->features = wp_tdmv_ec_features;
 
-	wan_set_bit(channel, &card->wandev.rtp_tap_call_map);
-	wp_fax_tone_timeout_set(wp, channel);
+	wan_set_bit(chan->chanpos-1, &card->wandev.rtp_tap_call_map);
+	wp_fax_tone_timeout_set(wp, chan->chanpos-1);
 
 	if (card->wandev.ec_enable) {
 		DEBUG_TDMV("[TDMV] %s: Enable HW echo canceller on channel %d\n",
@@ -2056,7 +2063,6 @@ static void wp_tdmv_hwec_free(struct dahdi_chan *chan, struct dahdi_echocan_stat
 {
 	wp_tdmv_softc_t	*wp = NULL;
 	sdla_t *card = NULL;
-	int	channel;
 
 	memset(ec, 0, sizeof(*ec));
 
@@ -2066,18 +2072,17 @@ static void wp_tdmv_hwec_free(struct dahdi_chan *chan, struct dahdi_echocan_stat
 	if(wp->card == NULL) return;
 	card = wp->card;
 
-	channel = chan->chanpos;
-	wan_clear_bit(channel, &card->wandev.rtp_tap_call_map);
+	wan_clear_bit(chan->chanpos-1, &card->wandev.rtp_tap_call_map);
 
 	if (card->wandev.ec_enable) {
 		DEBUG_TDMV("[TDMV] %s: Disable HW echo canceller on channel %d\n",
 				wp->devname, channel);
 
 		/* The ec persist flag enables and disables
-	         * persistent echo control.  In persist mode
-                 * echo cancellation is enabled regardless of
-                 * asterisk.  In persist mode off asterisk 
-                 * controls hardware echo cancellation */
+		 * persistent echo control.  In persist mode
+		 * echo cancellation is enabled regardless of
+		 * asterisk.  In persist mode off asterisk
+		 * controls hardware echo cancellation */
 		if (card->hwec_conf.persist_disable || IS_CHAN_HARDHDLC(chan)) {
 			card->wandev.ec_enable(card, 0, chan->chanpos);
 		}
@@ -2095,7 +2100,6 @@ static int wp_tdmv_hwec(struct zt_chan *chan, int enable)
 {
 	wp_tdmv_softc_t *wp = NULL;
 	sdla_t      *card = NULL;
-	int     channel;
 	int     err = -EINVAL;
 
 	WAN_ASSERT2(chan == NULL, -ENODEV);
@@ -2104,16 +2108,11 @@ static int wp_tdmv_hwec(struct zt_chan *chan, int enable)
 	WAN_ASSERT2(wp->card == NULL, -ENODEV);
 	card = wp->card;
 
-	channel = chan->chanpos;
-	if (!wp->ise1){
-		channel--;
-	}
-
 	if (enable) {
-		wan_set_bit(channel,&card->wandev.rtp_tap_call_map);
-		wp_fax_tone_timeout_set(wp,channel);
+		wan_set_bit(chan->chanpos-1,&card->wandev.rtp_tap_call_map);
+		wp_fax_tone_timeout_set(wp,chan->chanpos-1);
 	} else {
-		wan_clear_bit(channel,&card->wandev.rtp_tap_call_map);
+		wan_clear_bit(chan->chanpos-1,&card->wandev.rtp_tap_call_map);
 	}
 
 	if (card->wandev.ec_enable){
@@ -2148,13 +2147,17 @@ static int wp_tdmv_open(struct zt_chan *chan)
 {
 	wp_tdmv_softc_t	*wp = NULL;
 	sdla_t		*card = NULL;
+	wan_smp_flag_t flags;
 	
 	WAN_ASSERT2(chan == NULL, -ENODEV);
 	WAN_ASSERT2(chan->pvt == NULL, -ENODEV);
 	wp = chan->pvt;
 	WAN_ASSERT2(wp->card == NULL, -ENODEV);
 	card = wp->card;
+	wan_spin_lock_irq(&wp->lockirq, &flags);
 	wp->usecount++;
+	wan_spin_unlock_irq(&wp->lockirq, &flags);
+	wanpipe_open(card);
 	DEBUG_TDMV("%s: Open (usecount=%d, channo=%d, chanpos=%d)...\n", 
 				wp->devname,
 				wp->usecount,
@@ -2172,11 +2175,18 @@ static int wp_tdmv_open(struct zt_chan *chan)
 static int wp_tdmv_close(struct zt_chan *chan)
 {
 	wp_tdmv_softc_t*	wp = NULL;
-	
+	sdla_t		*card = NULL;
+	wan_smp_flag_t flags;
+
 	WAN_ASSERT2(chan == NULL, -ENODEV);
 	WAN_ASSERT2(chan->pvt == NULL, -ENODEV);
 	wp = chan->pvt;
+	WAN_ASSERT2(wp->card == NULL, -ENODEV);
+	card = wp->card;
+	wan_spin_lock_irq(&wp->lockirq, &flags);
 	wp->usecount--;
+	wan_spin_unlock_irq(&wp->lockirq, &flags);
+	wanpipe_close(card);
 	DEBUG_TDMV("%s: Close (usecount=%d, channo=%d, chanpos=%d)...\n", 
 				wp->devname,
 				wp->usecount,
@@ -2530,22 +2540,31 @@ static int wp_tdmv_rx_chan(wan_tdmv_t *wan_tdmv, int channo,
 }
 
 
-static int wp_tdmv_span_buf_rotate(void *pcard, u32 buf_sz, unsigned long mask)
+#undef AFT_TDM_ROTATE_DEBUG
+#ifdef AFT_TDM_ROTATE_DEBUG
+static int rotate_debug[32];
+#warning "AFT_TDM_ROTATE_DEBUG Enabled"
+#include <linux/sdla_aft_te1.h>
+#endif
+static int wp_tdmv_span_buf_rotate(void *pcard, u32 buf_sz, unsigned long mask, int circ_buf_len)
 {
 	sdla_t		*card = (sdla_t*)pcard;
 	wan_tdmv_t	*wan_tdmv = &card->wan_tdmv;
 	wp_tdmv_softc_t	*wp = NULL;
 	int x;
+	int tidx;
 	unsigned int rx_offset, tx_offset;
 	void *ptr;
-	
+
 	WAN_ASSERT(wan_tdmv->sc == NULL);
 	wp = wan_tdmv->sc; 
 
-	rx_offset = buf_sz * card->u.aft.tdm_rx_dma_toggle;
-	tx_offset = buf_sz * card->u.aft.tdm_tx_dma_toggle;
+#ifdef AFT_TDM_ROTATE_DEBUG
+	rotate_debug[card->tdmv_conf.span_no]++;
+#endif
 
 	for (x = 0; x < 32; x ++) {
+
 		if (wan_test_bit(x,&wp->timeslot_map)) {
 
 			if (card->u.aft.tdmv_dchan &&
@@ -2553,18 +2572,66 @@ static int wp_tdmv_span_buf_rotate(void *pcard, u32 buf_sz, unsigned long mask)
 				continue;
 			}
 
+			if (wp->ise1) {
+				tidx=x+1;
+			} else {
+				tidx=x;
+			}
+
+#ifdef AFT_TDM_ROTATE_DEBUG
+			if (1) {
+				u32 dma_descr, lo_reg, lo_reg_tx, lch;
+				lch=x;
+
+				dma_descr=(lch<<4) +
+							AFT_PORT_REG(card,AFT_RX_DMA_LO_DESCR_BASE_REG);
+				card->hw_iface.bus_read_4(card->hw,dma_descr,&lo_reg);
+			
+				dma_descr=(lch<<4) +
+							AFT_PORT_REG(card,AFT_TX_DMA_LO_DESCR_BASE_REG);
+				card->hw_iface.bus_read_4(card->hw,dma_descr,&lo_reg_tx);
+			
+				lo_reg=(lo_reg&AFT_TDMV_BUF_MASK)/AFT_TDMV_CIRC_BUF;
+				lo_reg_tx=(lo_reg_tx&AFT_TDMV_BUF_MASK)/AFT_TDMV_CIRC_BUF;
+
+#if 0
+				if (rotate_debug[card->tdmv_conf.span_no] < 5) {
+#else
+				if (rotate_debug[card->tdmv_conf.span_no] < 5 ||
+					abs(card->u.aft.tdm_rx_dma_toggle[tidx] - lo_reg) != 2 ||
+					abs(card->u.aft.tdm_tx_dma_toggle[tidx] - lo_reg_tx) != 2) {
+#endif
+					DEBUG_EVENT("%s: Rotating buffer lch=%i tidx=%i  rxdma=%i (hw=%i), txdma=%i (hw=%i)\n",
+						card->devname, lch, tidx, card->u.aft.tdm_rx_dma_toggle[tidx], lo_reg,card->u.aft.tdm_tx_dma_toggle[tidx],lo_reg_tx);
+				}
+			}
+#endif
+
+			rx_offset = buf_sz * card->u.aft.tdm_rx_dma_toggle[tidx];
+			tx_offset = buf_sz * card->u.aft.tdm_tx_dma_toggle[tidx];	
+
+			card->u.aft.tdm_rx_dma_toggle[tidx]++;
+			if (card->u.aft.tdm_rx_dma_toggle[tidx] >= circ_buf_len) {
+				card->u.aft.tdm_rx_dma_toggle[tidx]=0;
+			}
+			card->u.aft.tdm_tx_dma_toggle[tidx]++;
+			if (card->u.aft.tdm_tx_dma_toggle[tidx] >= circ_buf_len) {
+				card->u.aft.tdm_tx_dma_toggle[tidx]=0;
+			}
+
 			wan_spin_lock(&wp->chans[x].lock);
-				
-                        ptr=(void*)((((unsigned long)wp->chans[x].readchunk) & ~(mask)) + rx_offset);
+
+			ptr=(void*)((((unsigned long)wp->chans[x].readchunk) & ~(mask)) + rx_offset);
 			wp->chans[x].readchunk = ptr;
-                        ptr=(void*)((((unsigned long)wp->chans[x].writechunk) & ~(mask)) + tx_offset);
+
+			ptr=(void*)((((unsigned long)wp->chans[x].writechunk) & ~(mask)) + tx_offset);
 			wp->chans[x].writechunk = ptr;
 			
 			wan_spin_unlock(&wp->chans[x].lock);
 
 #if defined(__LINUX__)
-      			prefetch(wp->chans[x].readchunk);
-      			prefetch(wp->chans[x].writechunk);
+      		prefetch(wp->chans[x].readchunk);
+      		prefetch(wp->chans[x].writechunk);
 #endif
 
 			if (card->wandev.rtp_len && card->wandev.rtp_tap) {
@@ -2652,15 +2719,10 @@ static void wp_tdmv_callback_dtmf (void* card_id, wan_event_t *event)
 	sdla_t			*card = (sdla_t*)card_id;
 	wan_tdmv_t      *wan_tdmv = &card->wan_tdmv;
 	wp_tdmv_softc_t	*wp = NULL;
-	int fechan = 	event->channel;
+	int fechan = event->channel-1;
 
 	WAN_ASSERT1(wan_tdmv->sc == NULL);
 	wp = wan_tdmv->sc;
-
-	/* fechan check is a sanity check */
-	if (!wp->ise1 && fechan > 0){
-		fechan--;
-	}
 	
 	if (event->type != WAN_EVENT_EC_DTMF){
 		DEBUG_EVENT("ERROR: %s: Invalid event type %X!\n",
